﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace Combinator
{
    public static class Logger
    {
        public const int Information = 0;
        public const int DetailInformation = 1;
        public const int Warning = 2;
        public const int Error = 3;
        public const int FatalError = 4;

        #region Information Writing

        public static void WriteInformation(this ILogger logger, params object[] messages)
        {
            logger.Write(Logger.Information, messages);
        }

        public static void WriteDetailInformation(this ILogger logger, params object[] messages)
        {
            logger.Write(Logger.DetailInformation, messages);
        }

        public static void WriteWarning(this ILogger logger, params object[] messages)
        {
            logger.Write(Logger.Warning, messages);
        }

        public static void WriteError(this ILogger logger, params object[] messages)
        {
            logger.Write(Logger.Error, messages);
        }

        public static void WriteFatalError(this ILogger logger, params object[] messages)
        {
            logger.Write(Logger.FatalError, messages);
        }

        public static void WriteException(this ILogger logger, Exception ex)
        {
            logger.WriteFatalError(ex);
        }

        #endregion

        #region Logger Factory

        public static ILogger WriteTo(TextWriter writer)
        {
            return new TextWriterLogger()
            {
                Writer = writer
            };
        }

        public static ILogger WriteToConsole()
        {
            return new ConsoleLogger();
        }

        public static ILogger Sequence(params ILogger[] loggers)
        {
            return new SequenceLogger()
            {
                Loggers = loggers
            };
        }

        public static ILogger If(this ILogger logger, Func<int, bool> condition)
        {
            return new FilterLogger()
            {
                Condition = condition,
                Logger = logger,
            };
        }

        public static ILogger WithTimestamp(this ILogger logger)
        {
            return new TimestampLogger()
            {
                Logger = logger
            };
        }

        public static ILogger WithExceptionDetail(this ILogger logger, bool printInnerException = true, bool printStackTrace = true)
        {
            return new ExceptionLogger()
            {
                PrintInnerException = printInnerException,
                PrintStackTrace = printStackTrace,
                Logger = logger
            };
        }

        public static ILogger AcceptLevel(this ILogger logger, params int[] levels)
        {
            return logger.If(l => levels.Contains(l));
        }

        public static ILogger RejectLevel(this ILogger logger, params int[] levels)
        {
            return logger.If(l => !levels.Contains(l));
        }

        #endregion
    }
}
